@@ -257,6 +257,7 @@ vAPI.SafeAnimationFrame = class {
     let timer;
 
     const send = function() {
+        if ( self.vAPI instanceof Object === false ) { return; }
         vAPI.messaging.send('scriptlets', {
             what: 'securityPolicyViolation',
             type: 'net',
@@ -301,7 +302,7 @@ vAPI.SafeAnimationFrame = class {
             timer = undefined;
         }
         document.removeEventListener('securitypolicyviolation', listener);
-        vAPI.shutdown.remove(stop);
+        if ( vAPI ) { vAPI.shutdown.remove(stop); }
     };
 
     document.addEventListener('securitypolicyviolation', listener);
@@ -467,7 +468,7 @@ vAPI.injectScriptlet = function(doc, text) {
     if ( !doc ) { return; }
     let script, url;
     try {
-        const blob = new self.Blob([ text ], { type: 'text/javascript' });
+        const blob = new self.Blob([ text ], { type: 'text/javascript; charset=utf-8' });
         url = self.URL.createObjectURL(blob);
         script = doc.createElement('script');
         script.async = false;
@@ -662,9 +663,9 @@ vAPI.DOMFilterer = class {
             const proceduralFilterer = this.proceduralFiltererInstance();
             if ( proceduralFilterer !== null ) {
                 for ( const json of this.convertedProceduralFilters ) {
-                    out.procedural.push(
-                        proceduralFilterer.createProceduralFilter(json)
-                    );
+                    const pfilter = proceduralFilterer.createProceduralFilter(json);
+                    pfilter.converted = true;
+                    out.procedural.push(pfilter);
                 }
             }
         }
@@ -1066,6 +1067,7 @@ vAPI.DOMFilterer = class {
     };
 
     const doSurvey = ( ) => {
+        if ( self.vAPI instanceof Object === false ) { return; }
         const t0 = performance.now();
         const hashes = [];
         const nodes = pendingNodes;
@@ -1296,7 +1298,7 @@ vAPI.DOMFilterer = class {
         const {
             noSpecificCosmeticFiltering,
             noGenericCosmeticFiltering,
-            scriptlets,
+            scriptletDetails,
         } = response;
 
         vAPI.noSpecificCosmeticFiltering = noSpecificCosmeticFiltering;
@@ -1320,10 +1322,12 @@ vAPI.DOMFilterer = class {
 
         // Library of resources is located at:
         // https://github.com/gorhill/uBlock/blob/master/assets/ublock/resources.txt
-        if ( scriptlets && typeof self.uBO_scriptletsInjected !== 'boolean' ) {
-            self.uBO_scriptletsInjected = true;
-            vAPI.injectScriptlet(document, scriptlets);
-            vAPI.injectedScripts = scriptlets;
+        if ( scriptletDetails && typeof self.uBO_scriptletsInjected !== 'string' ) {
+            self.uBO_scriptletsInjected = scriptletDetails.filters;
+            if ( scriptletDetails.mainWorld ) {
+                vAPI.injectScriptlet(document, scriptletDetails.mainWorld);
+                vAPI.injectedScripts = scriptletDetails.mainWorld;
+            }
         }
 
         if ( vAPI.domSurveyor ) {
@@ -1344,7 +1348,7 @@ vAPI.DOMFilterer = class {
         vAPI.messaging.send('contentscript', {
             what: 'retrieveContentScriptParameters',
             url: vAPI.effectiveSelf.location.href,
-            needScriptlets: typeof self.uBO_scriptletsInjected !== 'boolean',
+            needScriptlets: typeof self.uBO_scriptletsInjected !== 'string',
         }).then(response => {
             onResponseReady(response);
         });
