@@ -21,6 +21,28 @@
 
 'use strict';
 
+/******************************************************************************/
+
+import contextMenu from './contextmenu.js';
+import logger from './logger.js';
+import staticNetFilteringEngine from './static-net-filtering.js';
+import µb from './background.js';
+import webext from './webext.js';
+import { orphanizeString } from './text-utils.js';
+import { redirectEngine } from './redirect-engine.js';
+
+import {
+    sessionFirewall,
+    sessionSwitches,
+    sessionURLFiltering,
+} from './filtering-engines.js';
+
+import {
+    domainFromHostname,
+    hostnameFromURI,
+    isNetworkURI,
+} from './uri-utils.js';
+
 /*******************************************************************************
 
 A PageRequestStore object is used to store net requests in two ways:
@@ -30,19 +52,13 @@ To create a log of net requests
 
 **/
 
-{
-
-// start of private namespace
-// >>>>>
-
-/******************************************************************************/
-
-const µb = µBlock;
-
 /******************************************************************************/
 
 const NetFilteringResultCache = class {
     constructor() {
+        this.pruneTimer = vAPI.defer.create(( ) => {
+            this.prune();
+        });
         this.init();
     }
 
@@ -50,7 +66,6 @@ const NetFilteringResultCache = class {
         this.blocked = new Map();
         this.results = new Map();
         this.hash = 0;
-        this.timer = undefined;
         return this;
     }
 
@@ -98,10 +113,7 @@ const NetFilteringResultCache = class {
         this.blocked.clear();
         this.results.clear();
         this.hash = 0;
-        if ( this.timer !== undefined ) {
-            clearTimeout(this.timer);
-            this.timer = undefined;
-        }
+        this.pruneTimer.off();
     }
 
     prune() {
@@ -123,14 +135,7 @@ const NetFilteringResultCache = class {
     }
 
     pruneAsync() {
-        if ( this.timer !== undefined ) { return; }
-        this.timer = vAPI.setTimeout(
-            ( ) => {
-                this.timer = undefined;
-                this.prune();
-            },
-            this.shelfLife
-        );
+        this.pruneTimer.on(this.shelfLife);
     }
 
     lookupResult(fctxt) {
@@ -188,9 +193,8 @@ const FrameStore = class {
         this.clickToLoad = false;
         this.rawURL = frameURL;
         if ( frameURL !== undefined ) {
-            this.hostname = vAPI.hostnameFromURI(frameURL);
-            this.domain =
-                vAPI.domainFromHostname(this.hostname) || this.hostname;
+            this.hostname = hostnameFromURI(frameURL);
+            this.domain = domainFromHostname(this.hostname) || this.hostname;
         }
         // Evaluated on-demand
         // - 0b01: specific cosmetic filtering
@@ -207,24 +211,33 @@ const FrameStore = class {
         return null;
     }
 
+    updateURL(url) {
+        if ( typeof url !== 'string' ) { return; }
+        this.rawURL = url;
+        this.hostname = hostnameFromURI(url);
+        this.domain = domainFromHostname(this.hostname) || this.hostname;
+        this._cosmeticFilteringBits = undefined;
+    }
+
     getCosmeticFilteringBits(tabId) {
         if ( this._cosmeticFilteringBits !== undefined ) {
             return this._cosmeticFilteringBits;
         }
         this._cosmeticFilteringBits = 0b11;
         {
-            const result = µb.staticNetFilteringEngine.matchStringReverse(
+            const result = staticNetFilteringEngine.matchRequestReverse(
                 'specifichide',
                 this.rawURL
             );
-            if ( result !== 0 && µb.logger.enabled ) {
-                µBlock.filteringContext
+            if ( result !== 0 && logger.enabled ) {
+                µb.filteringContext
                     .duplicate()
                     .fromTabId(tabId)
                     .setURL(this.rawURL)
+                    .setDocOriginFromURL(this.rawURL)
                     .setRealm('network')
                     .setType('specifichide')
-                    .setFilter(µb.staticNetFilteringEngine.toLogData())
+                    .setFilter(staticNetFilteringEngine.toLogData())
                     .toLogger();
             }
             if ( result === 2 ) {
@@ -232,18 +245,19 @@ const FrameStore = class {
             }
         }
         {
-            const result = µb.staticNetFilteringEngine.matchStringReverse(
+            const result = staticNetFilteringEngine.matchRequestReverse(
                 'generichide',
                 this.rawURL
             );
-            if ( result !== 0 && µb.logger.enabled ) {
-                µBlock.filteringContext
+            if ( result !== 0 && logger.enabled ) {
+                µb.filteringContext
                     .duplicate()
                     .fromTabId(tabId)
                     .setURL(this.rawURL)
+                    .setDocOriginFromURL(this.rawURL)
                     .setRealm('network')
                     .setType('generichide')
-                    .setFilter(µb.staticNetFilteringEngine.toLogData())
+                    .setFilter(staticNetFilteringEngine.toLogData())
                     .toLogger();
             }
             if ( result === 2 ) {
@@ -331,12 +345,17 @@ const PageStore = class {
     constructor(tabId, details) {
         this.extraData = new Map();
         this.journal = [];
-        this.journalTimer = undefined;
         this.journalLastCommitted = this.journalLastUncommitted = -1;
         this.journalLastUncommittedOrigin = undefined;
         this.netFilteringCache = NetFilteringResultCache.factory();
         this.hostnameDetailsMap = new HostnameDetailsMap();
         this.counts = new CountDetails();
+        this.journalTimer = vAPI.defer.create(( ) => {
+            this.journalProcess();
+        });
+        this.largeMediaTimer = vAPI.defer.create(( ) => {
+            this.injectLargeMediaElementScriptlet();
+        });
         this.init(tabId, details);
     }
 
@@ -376,7 +395,6 @@ const PageStore = class {
         this.remoteFontCount = 0;
         this.popupBlockedCount = 0;
         this.largeMediaCount = 0;
-        this.largeMediaTimer = null;
         this.allowLargeMediaElementsRegex = undefined;
         this.extraData.clear();
 
@@ -391,6 +409,10 @@ const PageStore = class {
         // Evaluated on-demand
         this._noCosmeticFiltering = undefined;
 
+        // Remember if the webpage was potentially improperly filtered, for
+        // reporting purpose.
+        this.hasUnprocessedRequest = vAPI.net.hasUnprocessedRequest(tabId);
+
         return this;
     }
 
@@ -419,10 +441,7 @@ const PageStore = class {
         }
 
         // A new page is completely reloaded from scratch, reset all.
-        if ( this.largeMediaTimer !== null ) {
-            clearTimeout(this.largeMediaTimer);
-            this.largeMediaTimer = null;
-        }
+        this.largeMediaTimer.off();
         this.disposeFrameStores();
         this.init(this.tabId, details);
         return this;
@@ -436,15 +455,9 @@ const PageStore = class {
         this.netFilteringCache.empty();
         this.allowLargeMediaElementsUntil = Date.now();
         this.allowLargeMediaElementsRegex = undefined;
-        if ( this.largeMediaTimer !== null ) {
-            clearTimeout(this.largeMediaTimer);
-            this.largeMediaTimer = null;
-        }
+        this.largeMediaTimer.off();
         this.disposeFrameStores();
-        if ( this.journalTimer !== undefined ) {
-            clearTimeout(this.journalTimer);
-            this.journalTimer = undefined;
-        }
+        this.journalTimer.off();
         this.journal = [];
         this.journalLastUncommittedOrigin = undefined;
         this.journalLastCommitted = this.journalLastUncommitted = -1;
@@ -461,7 +474,7 @@ const PageStore = class {
         ) {
             return false;
         }
-        this.title = µb.orphanizeString(details.title.slice(0, 128));
+        this.title = orphanizeString(details.title.slice(0, 128));
         return true;
     }
 
@@ -476,6 +489,8 @@ const PageStore = class {
         return this.frames.get(frameId) || null;
     }
 
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1858
+    //   Mind that setFrameURL() can be called from navigation event handlers.
     setFrameURL(details) {
         let { frameId, url, parentFrameId } = details;
         if ( frameId === undefined ) { frameId = 0; }
@@ -492,6 +507,9 @@ const PageStore = class {
         frameStore = FrameStore.factory(url, parentFrameId);
         this.frames.set(frameId, frameStore);
         this.frameAddCount += 1;
+        if ( url.startsWith('about:') ) {
+            frameStore.updateURL(this.getEffectiveFrameURL({ frameId }));
+        }
         if ( (this.frameAddCount & 0b111111) === 0 ) {
             this.pruneFrames();
         }
@@ -555,18 +573,18 @@ const PageStore = class {
         if ( this._noCosmeticFiltering === undefined ) {
             this._noCosmeticFiltering = this.getNetFilteringSwitch() === false;
             if ( this._noCosmeticFiltering === false ) {
-                this._noCosmeticFiltering = µb.sessionSwitches.evaluateZ(
+                this._noCosmeticFiltering = sessionSwitches.evaluateZ(
                     'no-cosmetic-filtering',
                     this.tabHostname
                 ) === true;
-                if ( this._noCosmeticFiltering && µb.logger.enabled ) {
+                if ( this._noCosmeticFiltering && logger.enabled ) {
                     µb.filteringContext
                         .duplicate()
                         .fromTabId(this.tabId)
                         .setURL(this.rawURL)
                         .setRealm('cosmetic')
                         .setType('dom')
-                        .setFilter(µb.sessionSwitches.toLogData())
+                        .setFilter(sessionSwitches.toLogData())
                         .toLogger();
                 }
             }
@@ -600,7 +618,7 @@ const PageStore = class {
     getAllHostnameDetails() {
         if (
             this.hostnameDetailsMap.has(this.tabHostname) === false &&
-            µb.URI.isNetworkURI(this.rawURL)
+            isNetworkURI(this.rawURL)
         ) {
             this.hostnameDetailsMap.set(
                 this.tabHostname,
@@ -616,19 +634,22 @@ const PageStore = class {
             allFrames: true,
             runAt: 'document_idle',
         });
-        µb.contextMenu.update(this.tabId);
+        contextMenu.update(this.tabId);
     }
 
     temporarilyAllowLargeMediaElements(state) {
         this.largeMediaCount = 0;
-        µb.contextMenu.update(this.tabId);
+        contextMenu.update(this.tabId);
         if ( state ) {
             this.allowLargeMediaElementsUntil = 0;
             this.allowLargeMediaElementsRegex = undefined;
         } else {
             this.allowLargeMediaElementsUntil = Date.now();
         }
-        µb.scriptlets.injectDeep(this.tabId, 'load-large-media-all');
+        vAPI.tabs.executeScript(this.tabId, {
+            file: '/js/scriptlets/load-large-media-all.js',
+            allFrames: true,
+        });
     }
 
     // https://github.com/gorhill/uBlock/issues/2053
@@ -638,11 +659,7 @@ const PageStore = class {
         const hostname = fctxt.getHostname();
         if ( hostname === '' ) { return; }
         this.journal.push(hostname, result, fctxt.itype);
-        if ( this.journalTimer !== undefined ) { return; }
-        this.journalTimer = vAPI.setTimeout(
-            ( ) => { this.journalProcess(true); },
-            µb.hiddenSettings.requestJournalProcessPeriod
-        );
+        this.journalTimer.on(µb.hiddenSettings.requestJournalProcessPeriod);
     }
 
     journalAddRootFrame(type, url) {
@@ -651,12 +668,12 @@ const PageStore = class {
             if (
                 this.journalLastUncommitted !== -1 &&
                 this.journalLastUncommitted < this.journalLastCommitted &&
-                this.journalLastUncommittedOrigin === vAPI.hostnameFromURI(url)
+                this.journalLastUncommittedOrigin === hostnameFromURI(url)
             ) {
                 this.journalLastCommitted = this.journalLastUncommitted;
             }
         } else if ( type === 'uncommitted' ) {
-            const newOrigin = vAPI.hostnameFromURI(url);
+            const newOrigin = hostnameFromURI(url);
             if (
                 this.journalLastUncommitted === -1 ||
                 this.journalLastUncommittedOrigin !== newOrigin
@@ -665,23 +682,16 @@ const PageStore = class {
                 this.journalLastUncommittedOrigin = newOrigin;
             }
         }
-        if ( this.journalTimer !== undefined ) {
-            clearTimeout(this.journalTimer);
-        }
-        this.journalTimer = vAPI.setTimeout(
-            ( ) => { this.journalProcess(true); },
-            µb.hiddenSettings.requestJournalProcessPeriod
-        );
+        this.journalTimer.offon(µb.hiddenSettings.requestJournalProcessPeriod);
     }
 
-    journalProcess(fromTimer = false) {
-        if ( fromTimer === false ) { clearTimeout(this.journalTimer); }
-        this.journalTimer = undefined;
+    journalProcess() {
+        this.journalTimer.off();
 
         const journal = this.journal;
         const pivot = Math.max(0, this.journalLastCommitted);
         const now = Date.now();
-        const { SCRIPT, SUB_FRAME } = µb.FilteringContext;
+        const { SCRIPT, SUB_FRAME, OBJECT } = µb.FilteringContext;
         let aggregateAllowed = 0;
         let aggregateBlocked = 0;
 
@@ -699,7 +709,7 @@ const PageStore = class {
             if ( itype === SCRIPT ) {
                 hnDetails.counts.inc(blocked, 'script');
                 this.counts.inc(blocked, 'script');
-            } else if ( itype === SUB_FRAME ) {
+            } else if ( itype === SUB_FRAME || itype === OBJECT ) {
                 hnDetails.counts.inc(blocked, 'frame');
                 this.counts.inc(blocked, 'frame');
             } else {
@@ -780,34 +790,34 @@ const PageStore = class {
         }
 
         const requestType = fctxt.type;
-        const loggerEnabled = µb.logger.enabled;
+        const loggerEnabled = logger.enabled;
 
         // Dynamic URL filtering.
-        let result = µb.sessionURLFiltering.evaluateZ(
+        let result = sessionURLFiltering.evaluateZ(
             fctxt.getTabHostname(),
             fctxt.url,
             requestType
         );
         if ( result !== 0 && loggerEnabled ) {
-            fctxt.filter = µb.sessionURLFiltering.toLogData();
+            fctxt.filter = sessionURLFiltering.toLogData();
         }
 
         // Dynamic hostname/type filtering.
         if ( result === 0 && µb.userSettings.advancedUserEnabled ) {
-            result = µb.sessionFirewall.evaluateCellZY(
+            result = sessionFirewall.evaluateCellZY(
                 fctxt.getTabHostname(),
                 fctxt.getHostname(),
                 requestType
             );
             if ( result !== 0 && result !== 3 && loggerEnabled ) {
-                fctxt.filter = µb.sessionFirewall.toLogData();
+                fctxt.filter = sessionFirewall.toLogData();
             }
         }
 
         // Static filtering has lowest precedence.
-        const snfe = µb.staticNetFilteringEngine;
+        const snfe = staticNetFilteringEngine;
         if ( result === 0 || result === 3 ) {
-            result = snfe.matchString(fctxt);
+            result = snfe.matchRequest(fctxt);
             if ( result !== 0 ) {
                 if ( loggerEnabled ) {
                     fctxt.setFilter(snfe.toLogData());
@@ -869,19 +879,19 @@ const PageStore = class {
 
         if ( this.getNetFilteringSwitch(fctxt) === false ) { return 0; }
 
-        let result = µb.staticNetFilteringEngine.matchHeaders(fctxt, headers);
+        let result = staticNetFilteringEngine.matchHeaders(fctxt, headers);
         if ( result === 0 ) { return 0; }
 
-        const loggerEnabled = µb.logger.enabled;
+        const loggerEnabled = logger.enabled;
         if ( loggerEnabled ) {
-            fctxt.filter = µb.staticNetFilteringEngine.toLogData();
+            fctxt.filter = staticNetFilteringEngine.toLogData();
         }
 
         // Dynamic filtering allow rules
         // URL filtering
         if (
             result === 1 &&
-            µb.sessionURLFiltering.evaluateZ(
+            sessionURLFiltering.evaluateZ(
                 fctxt.getTabHostname(),
                 fctxt.url,
                 fctxt.type
@@ -889,14 +899,14 @@ const PageStore = class {
         ) {
             result = 2;
             if ( loggerEnabled ) {
-                fctxt.filter = µb.sessionURLFiltering.toLogData();
+                fctxt.filter = sessionURLFiltering.toLogData();
             }
         }
         // Hostname filtering
         if (
             result === 1 &&
             µb.userSettings.advancedUserEnabled &&
-            µb.sessionFirewall.evaluateCellZY(
+            sessionFirewall.evaluateCellZY(
                 fctxt.getTabHostname(),
                 fctxt.getHostname(),
                 fctxt.type
@@ -904,7 +914,7 @@ const PageStore = class {
         ) {
             result = 2;
             if ( loggerEnabled ) {
-                fctxt.filter = µb.sessionFirewall.toLogData();
+                fctxt.filter = sessionFirewall.toLogData();
             }
         }
 
@@ -912,21 +922,24 @@ const PageStore = class {
     }
 
     redirectBlockedRequest(fctxt) {
-        const directives = µb.staticNetFilteringEngine.redirectRequest(fctxt);
+        const directives = staticNetFilteringEngine.redirectRequest(
+            redirectEngine,
+            fctxt
+        );
         if ( directives === undefined ) { return; }
-        if ( µb.logger.enabled !== true ) { return; }
+        if ( logger.enabled !== true ) { return; }
         fctxt.pushFilters(directives.map(a => a.logData()));
         if ( fctxt.redirectURL === undefined ) { return; }
         fctxt.pushFilter({
             source: 'redirect',
-            raw: µb.redirectEngine.resourceNameRegister
+            raw: redirectEngine.resourceNameRegister
         });
     }
 
     redirectNonBlockedRequest(fctxt) {
-        const directives = µb.staticNetFilteringEngine.filterQuery(fctxt);
+        const directives = staticNetFilteringEngine.filterQuery(fctxt);
         if ( directives === undefined ) { return; }
-        if ( µb.logger.enabled !== true ) { return; }
+        if ( logger.enabled !== true ) { return; }
         fctxt.pushFilters(directives.map(a => a.logData()));
         if ( fctxt.redirectURL === undefined ) { return; }
         fctxt.pushFilter({
@@ -937,13 +950,13 @@ const PageStore = class {
 
     filterCSPReport(fctxt) {
         if (
-            µb.sessionSwitches.evaluateZ(
+            sessionSwitches.evaluateZ(
                 'no-csp-reports',
                 fctxt.getHostname()
             )
         ) {
-            if ( µb.logger.enabled ) {
-                fctxt.filter = µb.sessionSwitches.toLogData();
+            if ( logger.enabled ) {
+                fctxt.filter = sessionSwitches.toLogData();
             }
             return 1;
         }
@@ -955,13 +968,13 @@ const PageStore = class {
             this.remoteFontCount += 1;
         }
         if (
-            µb.sessionSwitches.evaluateZ(
+            sessionSwitches.evaluateZ(
                 'no-remote-fonts',
                 fctxt.getTabHostname()
             ) !== false
         ) {
-            if ( µb.logger.enabled ) {
-                fctxt.filter = µb.sessionSwitches.toLogData();
+            if ( logger.enabled ) {
+                fctxt.filter = sessionSwitches.toLogData();
             }
             return 1;
         }
@@ -975,15 +988,15 @@ const PageStore = class {
         }
         if (
             netFiltering === false ||
-            µb.sessionSwitches.evaluateZ(
+            sessionSwitches.evaluateZ(
                 'no-scripting',
                 fctxt.getTabHostname()
             ) === false
         ) {
             return 0;
         }
-        if ( µb.logger.enabled ) {
-            fctxt.filter = µb.sessionSwitches.toLogData();
+        if ( logger.enabled ) {
+            fctxt.filter = sessionSwitches.toLogData();
         }
         return 1;
     }
@@ -1012,7 +1025,7 @@ const PageStore = class {
             return 0;
         }
         if (
-            µb.sessionSwitches.evaluateZ(
+            sessionSwitches.evaluateZ(
                 'no-large-media',
                 fctxt.getTabHostname()
             ) !== true
@@ -1025,15 +1038,10 @@ const PageStore = class {
         }
 
         this.largeMediaCount += 1;
-        if ( this.largeMediaTimer === null ) {
-            this.largeMediaTimer = vAPI.setTimeout(( ) => {
-                this.largeMediaTimer = null;
-                this.injectLargeMediaElementScriptlet();
-            }, 500);
-        }
+        this.largeMediaTimer.on(500);
 
-        if ( µb.logger.enabled ) {
-            fctxt.filter = µb.sessionSwitches.toLogData();
+        if ( logger.enabled ) {
+            fctxt.filter = sessionSwitches.toLogData();
         }
 
         return 1;
@@ -1062,14 +1070,14 @@ const PageStore = class {
             }
         }
         if ( exceptCname === undefined ) {
-            const result = µb.staticNetFilteringEngine.matchStringReverse(
+            const result = staticNetFilteringEngine.matchRequestReverse(
                 'cname',
                 frameStore instanceof Object
                     ? frameStore.rawURL
                     : fctxt.getDocOrigin()
             );
             exceptCname = result === 2
-                ? µb.staticNetFilteringEngine.toLogData()
+                ? staticNetFilteringEngine.toLogData()
                 : false;
             if ( frameStore instanceof Object ) {
                 frameStore.exceptCname = exceptCname;
@@ -1083,7 +1091,7 @@ const PageStore = class {
     }
 
     getBlockedResources(request, response) {
-        const normalURL = µb.normalizePageURL(this.tabId, request.frameURL);
+        const normalURL = µb.normalizeTabURL(this.tabId, request.frameURL);
         const resources = request.resources;
         const fctxt = µb.filteringContext;
         fctxt.fromTabId(this.tabId)
@@ -1121,11 +1129,6 @@ PageStore.prototype.collapsibleResources = new Set([
 PageStore.junkyard = [];
 PageStore.junkyardMax = 10;
 
-µb.PageStore = PageStore;
-
 /******************************************************************************/
 
-// <<<<<
-// end of private namespace
-
-}
+export { PageStore };
